#!/usr/bin/perl 
use strict;
use warnings;
use Getopt::Long;
use POSIX qw/strftime/;
use File::Basename;

my $basedir="/home/thomas/Klausuren/";
my $outputdir="/var/www/fsi-hp/fileadmin/klau/";
my $zipcmd='7za a -tzip -r \'-x!.svn*\' ';
my ($all, $sem, $sub, $help, $nochangelog, $date, $list, $complete) = 0;
my (@svndump, @faillog, @change, @sems, @subjects);
my %tochange;
my $args = @ARGV;

$outputdir .= "/" if (substr($outputdir, length($outputdir)-1, 1) ne "/");
$basedir .= "/" if (substr($basedir, length($basedir)-1, 1) ne "/");

sub process_sem($);
sub process_sub($$);
sub raise_error($);
sub mkcomplete($);
sub writelog();


##### init
Getopt::Long::Configure ("bundling", "gnu_compat");
GetOptions ("all|a" => \$all,
			"help|h" => \$help,
			"nochangelog|n" => \$nochangelog,
			"semester|s=i" => \@sems,
			"subject|u=s" => \@subjects,
			"complete|c=i" => \$complete,
			"list|l" => \$list) 
			or die "Invalid command-line!";

if($help){
	print "Usage: $0 [-ansuh] [x]\n" .
		"\t-a, --all\t\tAlles (wirklich alles!) inkl. des Changelogs neu erstellen, nur in Ausnahmefllen verwenden!\n" .
		"\t-n, --nochangelog\tDas Changelog nicht updaten.\n" .
		"\t-s, --semester x\tMuss zusammen mit -u verwendet werden\n" .
		"\t-u, --subject sub\tFach sub aus Semester x (siehe -s) aktualisieren\n" .
		"\t-l, --list\t\tEine Liste der verfgbaren Semester-/Fcherkombinationen ausgeben\n" . 
		"\t-c, --complete x\tnur \"Komplettarchiv\" fuer Semester x erstellen\n" . 
		"\t-h, --help\t\tDiese Hilfe anzeigen\n" .
		"Ohne commandline-switches knnen einzelne Semester als Argumente bergeben werden\n";
		exit 0;
}
if($list){
	foreach my $num(1..6){
			print "Semester $num:\n";
		foreach my $dir_(glob($basedir . $num . "_semester/*")){
			next if (! -d $dir_ || $dir_ =~ m#\.svn#);
			print("\t" . basename($dir_) . "\n");
		}
		print "-" x 15 . "\n";
	}
	exit 0;
}

$date = strftime('%Y-%m-%d', localtime); # e.g. 2009-07-23
open my $logfile, ">", "/var/log/svnlog/faillog_" . $date . ".log" or die $!;
##every error below this line should be logged in automatic mode

@svndump = `svn status -u "$basedir"` or 
			raise_error("svn status -u failed");
open my $changelog, "<", $outputdir . "changelog" or 
			raise_error("Couldn't open changelog: " . $outputdir . "changelog");
@change = <$changelog>;
unshift @change, "-" x 15 . "\n";
close $changelog;

#### end init

if (@svndump == 1 && ! $all && ! @sems && ! @subjects && ! $args && !$complete){
	print $logfile "-" x 15 . "\nEverything is up-to-date!\n";
	close $logfile;
	exit 0;
}
else {
	`svn update "$basedir"` or raise_error("svn update failed");
}
#running in automatic mode
if ($args == 0 && ! $all && ! @sems && !@subjects && ! $complete ){
   	$_ =~ s#.*([1-6]_semester/[^/]+)(/|$).*#$1# foreach(@svndump);
	@svndump = grep /[1-6]_semester\//, @svndump;
	my %all=();
	@all{@svndump}=1;
	@svndump = sort {(split //, $a)[0] <=> (split //,$b)[0]} (keys %all);
	chomp @svndump;

	foreach my $line (@svndump){
		$sem = (split //, $line)[0];
		($sub = $line) =~ s#.*/([^/]+)$#$1#; ##
		push @{$tochange{$sem}}, $sub;
	}
		foreach $sem (keys %tochange){
			mkcomplete($sem);
			foreach $sub (@{$tochange{$sem}}){
				process_sub($sem, $sub);
			}
		}
}
else {
	print $logfile "-" x 15 . "\nRunning in interactive log - observe the shell window!\n";
	if($complete){
		mkcomplete($complete);
	}
	if (@sems > 0 || @subjects >0){ #both are necessary for error detection
		if (scalar @subjects != scalar @sems){
			raise_error"number of subjects != number of terms...\n";
		}
		for (my $i=0; $i<@sems; $i++){
			process_sub($sems[$i], $subjects[$i]);
		}
	}
	#update everything
	elsif ($all){
		foreach (1..6){
			process_sem($_);
		}
	}
	else{
		$all = 1;
		foreach (@ARGV){
			process_sem($_);
		}
	}
}

unless($nochangelog){
	open $changelog, ">", $outputdir . "changelog" or die $!;
	print $changelog @change;
	close $changelog;
}
close $logfile;
### end ###




### subs... ###

#update subject $sub from term $sem
sub process_sub($$){
	$sem = shift; # NAUE
	$sub = shift; # 6
	unshift @change, "$date\t$sub aus dem $sem. Semester aktualisiert.\n";
	
	my $sub_filename = $sub; # NAUE
	$sub_filename =~ y#A-Z #a-z #; # naue
	$sub_filename =~ s#[^0-9a-z_]##g;
	my $dir = $outputdir . $sem . "_semester/"; #home/thomas/klau/6_semester/
	my $zipname = $dir . $sub_filename . ".zip";#home/thomas/klau/6_semester/naue.zip
	my $toprocess = $basedir . $sem . "_semester/" . $sub; #/home/thomas/Klausuren/6_semester/NAUE/

	raise_error("directory $toprocess not found") if (! -e $toprocess);
	my $cmd = $zipcmd .  "'$zipname'" . " " . "'$toprocess'";
	@faillog = `$cmd`;
	raise_error("process_sub(sem = $sem, sub = $sub) failed zipcmd = $cmd") if($?);
	write_log();
	chown 33, 33, $zipname;
	push @faillog, "-" x 15 . "\n";
#	7za a -tzip -r "-x!.svn*" "$OUTPUTDIR"/"$SEM"_semester/"$archivename" "$line"

}

#update every archive of semester x
sub process_sem($){
	$sem = shift;
	raise_error("$sem: invalid argument") if $sem !~ m#[1-6]# ;
	mkcomplete($sem) if ($all);
	foreach my $line(glob($basedir . $sem . "_semester/*")){
		next if (! -d $line or $line =~m#\.svn#);
		process_sub($sem, basename($line));
	}
}

sub mkcomplete($){
	$sem = shift;
	unshift @change, "$date\t $sem. Semester aktualisiert.\n";
	raise_error("$sem: invalid argument")	if $sem !~ m#[1-6]#;
	my $dir = $outputdir . $sem . "_semester/"; #home/thomas/klau/6_semester/
	my $zipname = $dir . $sem . "_semester_komplett.zip";#home/thomas/klau/6_semester/naue.zip
	$dir = $basedir . $sem . "_semester";
	my $cmd = $zipcmd . "'$zipname'" . " " . "'$dir'";
	@faillog = `$cmd`;
	raise_error("zimpcmd failed: $cmd") if($?);
	write_log();
	chown 33, 33, $zipname;
	push @faillog, "-" x 15 . "\n";
}

sub raise_error($){
	my $errmsg = shift;
	my $errdate = localtime(time);
	if($args == 0){
		write_log();
		print $logfile "--- $errdate ---\n";
		print $logfile "$errmsg\n";
	}
	else {
		print @faillog;
		print "--- Error ---\n";
		print "$errmsg\n";
	}
	close $logfile;
	#send mail to admin....
	exit 1;
}

sub write_log(){
	if ($args == 0){
		print $logfile @faillog;
	}else{
		print @faillog;
	}
}
